#ifndef Synthesizer_hpp
#define Synthesizer_hpp

#include "Encoding.hpp"
#include "Keywords.hpp"

namespace SoftWire
{
	typedef unsigned char byte;

	class Instruction;

	class Synthesizer
	{
	public:
		Synthesizer();

		~Synthesizer();

		void reset();

		void defineLabel(const char *label);
		void referenceLabel(const char *label);

		void encodeFirstOperand(const Operand &firstOperand);
		void encodeSecondOperand(const Operand &secondOperand);
		void encodeThirdOperand(const Operand &thirdOperand);
		
		void encodeBase(const Operand &base);
		void encodeIndex(const Operand &index);
		void encodeScale(int scale);

		void encodeDisplacement(int displacement);
		void encodeImmediate(int i);

		const Encoding &encodeInstruction(const Instruction *instruction);

	private:
		Encoding encoding;

		Operand::Type firstType;
		Operand::Type secondType;

		Encoding::Reg firstReg;
		Encoding::Reg secondReg;
		Encoding::Reg baseReg;
		Encoding::Reg indexReg;

		int scale;

		void encodeModField();
		void encodeSibByte();
	};
}

#endif   // Synthesizer_hpp